#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 17-May-2003
#################################################################################
#
# Configure Cookie - Script part of Ethereal Realms designed to control cookies
# Copyright (C) 2000-2003  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204, 817 5TH ST NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use eRealms::Crypto;						# Crypto library
use eRealms::Database;						# Database handler
use eRealms::Login;						# Login functionality
use eRealms::Param;						# Parameter control


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle

my %config;							# Configuration tag

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;	
	$database = new eRealms::Database();

	# Connect and fetch
	$database->Connect();

	# Parameter handling 
	$param = new eRealms::Param($database, $cgi);
	$param->GetParam();

	# Connect to hash
	$database->GetHashConfiguration(\%config);


	# Document header
	if (CookieHandler($database, $cgi, $param, \%config))
	{
		# Document header
		$database->DocumentGetHeader();


		# Login handle
		$login = new eRealms::Login($database, $cgi, $param);

		# Authenthication
		if ($login->GetVerificationNormal())
		{
			# Front end display
			CookieDisplay($database, $cgi, $param, \%config);
		}

		# Document footer
		$database->DocumentGetFooter();
	}


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Cookie Handler
#
# Handles the addition and removal of Cookies.  This is of course based on the
# creation of a header, which is why it must be placed beforehand.

sub CookieHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Configuration hash

	my $cookie;						# Cookie
	my $mori;						# Kill switch

	my $domain;						# Domain of 
	my $inline;						# Inline parameters

	my $crypto   = new eRealms::Crypto();			# Crypto handle
	my $crypt;						# Crypted password

	my %system;						# System Hash


	#####################
	# Program area	

	# System Hash
	$database->GetHashSystem(\%system);


	# Fetch URL
	$domain = $cgi->url(-base=>1);

	# Redirection
	if ($domain !~ /$system{'SetDomain'}/)
	{
		# Generate inline
		$inline =  $param->EmbedInline($param->Flat());

		# Redirection header
		print $cgi->redirect($system{'LocScriptCookie'} . $inline);


		# Return false
		return 0;
	}



	# Assign Values
	# Kill switch after a month
	$mori = time + 6696000;

	# Encrypted password
	$crypt = $crypto->EncryptParam(
		USER  => $param->{'USER'}, 
		CRYPT => $param->{'CRYPT'},
		MORI  => $mori
	 );


	# Cookie Addition
	if (defined($param->{$config->{'TxtSubmitAddition'}}))
	{
		# Generate cookies
		$cookie = $cgi->cookie(
			 -NAME		=> 'ETHEREAL',
			 -VALUE		=> $crypt,
			 -DOMAIN	=> $system{'SetDomain'},
			 -EXPIRES 	=> "+1M"
			);


		# Cookie Enable Header
		print $cgi->header(-cookie=>$cookie);

		# Display Message
		$param->{'MESSAGE'} = $config->{'MsgCookieAdd'};
	}

	# Cookie Removal
	elsif (defined($param->{$config->{'TxtSubmitDelete'}}))
	{

		# Generate cookies
		$cookie = $cgi->cookie(
			 -NAME		=> 'ETHEREAL',
			 -VALUE		=> $crypt,
			 -DOMAIN	=> $system{'SetDomain'},
			 -EXPIRES 	=> "-1y"
			);

		# Cookie Enable Header
		print $cgi->header(-cookie=>$cookie);

		# Display Message
		$param->{'MESSAGE'} = $config->{'MsgCookieRem'};
	}

	# Standard header
	else
	{
		# Standard header
		print $cgi->header();
	}

	# Standard return
	return 1;
}


#####################
# Cookie Display
#
# Will handle the prompt for the addition and removal of cookies from Etherea 
# Realms.   This of course will be a rather straight forward display.

sub CookieDisplay
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Configuration hash

	my $add;						# Add button
	my $doc;						# Document display
	my $mes;						# Message display
	my $rem;						# Remove button



	#####################
	# Program area	

	# Generate Buttons
	$add = $cgi->submit($config->{'TxtSubmitAddition'});
	$rem = $cgi->submit($config->{'TxtSubmitDelete'});


	# Document
	$doc = $database->DocumentGetCookie();

	# Status message
	$mes = (defined($param->{'MESSAGE'})) ? $param->{'MESSAGE'} : '';

	# Clean mess
	$param->Cleanup('MESSAGE');


	# Replace for use
	$doc =~ s/MMESSAGES/$mes/gs;
	$doc =~ s/WCOOKMAKE/$add/gs;
	$doc =~ s/WCOOKPULL/$rem/gs;


	# Information display
	# Start form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat()), "\n";

	# Display
	print $doc . "\n";

	# Form Embed
	print $cgi->end_form();
}
